@@ -1,6 +1,6 @@
 ###############################################################################
 ##
-##  Copyright 2011,2012 Tavendo GmbH
+##  Copyright 2011-2013 Tavendo GmbH
 ##
 ##  Licensed under the Apache License, Version 2.0 (the "License");
 ##  you may not use this file except in compliance with the License.
@@ -16,6 +16,17 @@
 ##
 ###############################################################################
 
+__all__ = ("WampProtocol",
+           "WampFactory",
+           "WampServerProtocol",
+           "WampServerFactory",
+           "WampClientProtocol",
+           "WampClientFactory",
+           "WampCraProtocol",
+           "WampCraClientProtocol",
+           "WampCraServerProtocol",)
+
+
 import json
 import random
 import inspect, types
@@ -682,8 +693,13 @@ class WampServerProtocol(WebSocketServerProtocol, WampProtocol):
       :type exclude: list of obj
       :param eligible: Optional list of clients (WampServerProtocol instances) eligible at all (or None for all).
       :type eligible: list of obj
+
+      :returns twisted.internet.defer.Deferred -- Will be fired when event was
+      dispatched to all subscribers. The return value provided to the deferred
+      is a pair (delivered, requested), where delivered = number of actual
+      receivers, and requested = number of (subscribers - excluded) & eligible.
       """
-      self.factory.dispatch(topicUri, event, exclude, eligible)
+      return self.factory.dispatch(topicUri, event, exclude, eligible)
 
 
    def onMessage(self, msg, binary):
@@ -741,7 +757,8 @@ class WampServerProtocol(WebSocketServerProtocol, WampProtocol):
                                  self.factory._subscribeClient(self, topicUri)
                            except:
                               if self.debugWamp:
-                                 log.msg("execption during topic subscription handler")
+                                 log.msg("exception during topic subscription handler:")
+                              traceback.print_exc()
                      else:
                         if self.debugWamp:
                            log.msg("topic %s matches only by prefix and prefix match disallowed" % topicUri)
@@ -813,7 +830,8 @@ class WampServerProtocol(WebSocketServerProtocol, WampProtocol):
                                  self.factory.dispatch(topicUri, e, exclude, eligible)
                            except:
                               if self.debugWamp:
-                                 log.msg("execption during topic publication handler")
+                                 log.msg("exception during topic publication handler:")
+                              traceback.print_exc()
                      else:
                         if self.debugWamp:
                            log.msg("topic %s matches only by prefix and prefix match disallowed" % topicUri)
@@ -1485,7 +1503,7 @@ class WampCraProtocol(WampProtocol):
       http://en.wikipedia.org/wiki/PBKDF2.
 
       The function will only return a derived key if at least 'salt' is
-      present in the 'extra' dictionary. The complete set of attribtues
+      present in the 'extra' dictionary. The complete set of attributes
       that can be set in 'extra':
 
          salt: The salt value to be used.
@@ -1522,12 +1540,15 @@ class WampCraProtocol(WampProtocol):
       """
       if authSecret is None:
          authSecret = ""
+      if isinstance(authSecret, unicode):
+         authSecret = authSecret.encode('utf8')
       authSecret = WampCraProtocol.deriveKey(authSecret, authExtra)
       h = hmac.new(authSecret, authChallenge, hashlib.sha256)
       sig = binascii.b2a_base64(h.digest()).strip()
       return sig
 
 
+
 class WampCraClientProtocol(WampClientProtocol, WampCraProtocol):
    """
    Simple, authenticated WAMP client protocol.
@@ -1862,6 +1883,7 @@ class WampCraServerProtocol(WampServerProtocol, WampCraProtocol):
       return perms['permissions']
 
 
+
 class Call:
    """
    Thin-wrapper for incoming RPCs provided to call handlers registered via
@@ -2262,6 +2284,7 @@ class CallResultHandler(Handler):
             log.msg("callid not found for received call result message")
 
 
+
 class CallErrorHandler(Handler):
 
    typeid = WampProtocol.MESSAGE_TYPEID_CALL_ERROR
